﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows; //WPF Grafikához
using System.Windows.Controls; //WPF Grafikához
using System.Windows.Media; //WPF Grafikához

namespace Erettsegi14_Telek
{
    class telek
    {
        public int hsz; //házszám
        public int sz; //szélesség
        public int h; //hosszúság
        public bool js { get { return hsz % 2 == 0; } } //jólétsori telek-e?
        public bool gs { get { return !js; } } //gazdagsori telek-e?
        public int terület { get { return sz * h; } } //telek területe
        public int tk; //telek kezdete a merőleges utcától
        public int tv { get { return tk + sz; } } //telek vége a merőleges utcától
        public int adó // telek után fizetendő adó
        {
            get
            {
                double sadó = terület * 51;
                if (terület > 700) sadó = 700 * 51 + terület % 700 * 39;
                if (terület > 1000) sadó = 700 * 51 + 300 * 39 + 200;
                if (sz <= 15 || h <= 25) sadó *= 0.8;
                return (int)Math.Round(sadó / 100, 0) * 100;
            }
        }
    }

    class MyBtn : Button //Saját osztály a grafikához (nem feladat)
    {//Egyszerűség kedvéért a telkeket parancsgombok segítségével jelenítjük meg
        public MyBtn(int sz, int h, int hsz, int tk, bool gs) //konstruktor, ami felkészíti a "kirajzolást"
        {
            Width = sz * 3; Height = h * 3; //3x-os nagyítással rajzolunk
            Content = (gs ? "gs" : "js") + hsz + "\n" + sz + "x" + h; //Gombok felirata
            Margin = new Thickness(tk * 3, 0, 0, 0); //x pozíció
            HorizontalAlignment = HorizontalAlignment.Left;
            VerticalAlignment = gs ? VerticalAlignment.Bottom : VerticalAlignment.Top; //y pozíció (lent vagy fent)
            Random rnd = new Random(hsz); //Seed érték = házszám :-)
            byte r = (byte)rnd.Next(150, 256); //Világos színek végett 150-255
            byte g = (byte)rnd.Next(150, 256);
            byte b = (byte)rnd.Next(150, 256);
            Background = new SolidColorBrush(Color.FromRgb(r, g, b));
        }
    }


    class Erettsegi14_Telek
    {
        [STAThread] //Szálkezelési opció, a WPF grafikához kell
        //WPF Grafikához + Referenciák: PresentationCore, PresentationFramework, WindowsBase, System.Xaml kell hozzáadni
        static void Main()
        {
            Console.WriteLine("1. Feladat: telkek olvasása, adatszerkezet feltöltése");
            List<telek> telkek = new List<telek>();
            List<string> s = new List<string>(File.ReadAllLines("telkek.txt"));
            for (int i = 1; i < s.Count; i++) telkek.Add(new telek() { hsz = int.Parse(s[i].Split(' ')[0]), sz = int.Parse(s[i].Split(' ')[1]), h = int.Parse(s[i].Split(' ')[2]) });

            Console.WriteLine("2. Feladat: A séta hossza: {0} m", telkek.Sum(w => w.sz) + 160);

            Console.WriteLine("3. Feladat: Teljes beépítés a Jólétsoron: {0} db telek", telkek.Where(w => w.js && w.sz <= 20).Count());

            Console.WriteLine("4. Feladat: Távolság");
            var gazdagsor = telkek.Where(w => w.gs).OrderBy(x => x.terület); //Gazdagsori telkek, a méretük alapján sorbarendezve
            Console.WriteLine("{0} háznyi távolságra van egymástól Gazdagsor legnagyobb és legkisebb telke", (Math.Abs(gazdagsor.First().hsz - gazdagsor.Last().hsz) / 2) - 1);
            Console.WriteLine("Legnagyobb telek - házszám: {0}, terület: {1} m2", gazdagsor.Last().hsz, gazdagsor.Last().terület);
            Console.WriteLine("Legkisebb telek - házszám: {0}, terület: {1} m2", gazdagsor.First().hsz, gazdagsor.First().terület);

            Console.WriteLine("5. feladat: Adó a Gazdagsoron {0} fabatka.", telkek.Where(w => w.gs).Sum(x => x.adó));

            Console.WriteLine("6. feladat: Utolsó 3 telek a Jólétsoron");
            var jóléts = telkek.Where(w => w.js).OrderBy(x => x.hsz).ToArray(); //jólétsor telkei
            for (int i = 1; i < jóléts.Count(); i++) jóléts[i].tk = jóléts[i - 1].tk + jóléts[i - 1].sz; //távolságok az  utca elejétől
            for (int i = jóléts.Count() - 1; i > jóléts.Count() - 4; i--) Console.WriteLine("Házszám: {0} Táv: {1} m", jóléts[i].hsz, jóléts[i].tk);

            Console.WriteLine("7. Feladat: Jólétsor telkeinek hosszúsága");
            var gazdags = telkek.Where(w => w.gs).OrderBy(x => x.hsz).ToArray();  //gazdagsor telkei
            for (int i = 1; i < gazdags.Count(); i++) gazdags[i].tk = gazdags[i - 1].tk + gazdags[i - 1].sz; //távolságok az utca elejétől

            for (int i = 0; i < jóléts.Length; i++) //jólétsori telkek hossza = 70 - leghosszabb szembenlévő telek:
                jóléts[i].h = 70 - gazdags.Where(gs => (gs.tv >= jóléts[i].tk && gs.tv <= jóléts[i].tv) || //vége lóg bele
                                                       (gs.tk >= jóléts[i].tk && gs.tk <= jóléts[i].tv) || //eleje lóg bele
                                                       (gs.tk < jóléts[i].tk && gs.tv > jóléts[i].tv) //átfedő telek pl: 17, 27
                                                       ).Select(x => x.h).Max();
            List<string> ki = new List<string>();
            for (int i = 0; i < jóléts.Length; i++) ki.Add(String.Join(";", jóléts[i].hsz, jóléts[i].sz, jóléts[i].h));
            File.WriteAllLines("joletsor.csv", ki);

            //Itt a vége, kirajzolás nem feladat!

            //Rajzoljuk ki a telkeket 3x-os nagyítással:
            Window MyWin = new Window() { Width = 1156, Height = 260, Title = "Telkek és a Dirib patak" }; //ablak
            Grid MyG = new Grid() { Background = Brushes.Blue }; //Grid-re "rajzolunk"
            foreach (var i in jóléts.Union(gazdags)) MyG.Children.Add(new MyBtn(i.sz, i.h, i.hsz, i.tk, i.gs));
            MyWin.Content = MyG;
            MyWin.ShowDialog();
        }
    }
}
